// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/src/domain/theme_extensions/rich_text_builder_theme_extension.dart';

class RichTextBuilderThemeExtensionDefault
    extends RichTextBuilderThemeExtension {
  const RichTextBuilderThemeExtensionDefault({
    required super.defaultStyle,
    required super.styles,
  });

  /// Creates a [RichTextBuilderThemeExtensionDefault] from a [ThemeData]
  /// and opinionated defaults.
  ///
  /// {@macro card_theme_extension_default}
  factory RichTextBuilderThemeExtensionDefault.from(ThemeData theme) =>
      RichTextBuilderThemeExtensionDefault(
        defaultStyle: theme.textTheme.bodyMedium,
        styles: {
          'blue': theme.textTheme.bodyMedium?.copyWith(
                color: Colors.blue,
              ) ??
              const TextStyle(
                color: Colors.blue,
              ),
          'red': theme.textTheme.bodyMedium?.copyWith(
                color: Colors.red,
              ) ??
              const TextStyle(
                color: Colors.red,
              ),
          'green': theme.textTheme.bodyMedium?.copyWith(
                color: Colors.green,
              ) ??
              const TextStyle(
                color: Colors.green,
              ),
          'bold': theme.textTheme.bodyMedium?.copyWith(
                fontWeight: FontWeight.bold,
              ) ??
              const TextStyle(
                fontWeight: FontWeight.bold,
              ),
          'italic': theme.textTheme.bodyMedium?.copyWith(
                fontStyle: FontStyle.italic,
              ) ??
              const TextStyle(fontStyle: FontStyle.italic),
        },
      );

  /// Creates a [RichTextBuilderThemeExtensionDefault] from a dark [ThemeData]
  factory RichTextBuilderThemeExtensionDefault.dark() =>
      RichTextBuilderThemeExtensionDefault.from(ThemeData.dark());

  /// Creates a [RichTextBuilderThemeExtensionDefault] from a light [ThemeData]
  factory RichTextBuilderThemeExtensionDefault.light() =>
      RichTextBuilderThemeExtensionDefault.from(ThemeData.light());
}
