// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// super program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// super program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with super program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:wyatt_component_copy_with_extension/wyatt_component_copy_with_extension.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/wyatt_ui_kit.dart';
part 'top_app_bar.g.dart';

@ComponentCopyWithExtension()
class TopAppBar extends TopAppBarComponent with $TopAppBarCWMixin {
  const TopAppBar({
    super.title,
    super.centerTitle,
    super.systemOverlayStyle,
    super.automaticallyImplyLeading,
    super.flexibleSpace,
    super.bottom,
    super.elevation,
    super.scrolledUnderElevation,
    super.shadowColor,
    super.surfaceTintColor,
    super.backgroundColor,
    super.iconTheme,
    super.primary,
    super.excludeHeaderSemantics,
    super.toolbarHeight,
    super.leadingWidth,
    super.leading,
    super.actions,
    super.shape,
    super.expandedWidget,
    super.key,
  });

  @override
  Widget build(BuildContext context) => DecoratedBox(
        decoration: BoxDecoration(
          gradient: ThemeHelper.maybeGetElement<MultiColor, Gradient>(
            [
              backgroundColor,
              context.themeExtension<TopBarThemeExtension>()?.backgroundColors,
              TopBarThemeExtensionDefault.from(Theme.of(context))
                  .backgroundColors,
            ],
            valueValidator: (value) => value?.isGradient,
            transform: (value) =>
                GradientHelper.linearFromNullableColors(value?.colors),
          ),
          color: ThemeHelper.maybeGetElement<MultiColor, Color>(
            [
              backgroundColor,
              context.themeExtension<TopBarThemeExtension>()?.backgroundColors,
              TopBarThemeExtensionDefault.from(Theme.of(context))
                  .backgroundColors,
            ],
            valueValidator: (value) => value?.isColor,
            transform: (value) => value?.color,
          ),
        ),
        child: Column(
          children: [
            AppBar(
              titleSpacing: 0,
              backgroundColor: Colors.transparent,
              systemOverlayStyle: systemOverlayStyle,
              automaticallyImplyLeading: automaticallyImplyLeading ?? true,
              flexibleSpace: flexibleSpace,
              bottom: bottom,
              elevation: elevation ?? 0,
              scrolledUnderElevation: scrolledUnderElevation,
              shadowColor: shadowColor,
              surfaceTintColor: surfaceTintColor,
              iconTheme: ThemeHelper.getElement<IconThemeData, IconThemeData>(
                [
                  iconTheme,
                  context.themeExtension<TopBarThemeExtension>()?.iconTheme,
                  TopBarThemeExtensionDefault.from(Theme.of(context)).iconTheme,
                ],
              ),
              primary: primary ?? true,
              excludeHeaderSemantics: excludeHeaderSemantics ?? false,
              leadingWidth: leadingWidth,
              title: GradientText(
                data: title?.data ?? '',
                style: ThemeHelper.getElement<TextStyle, TextStyle>(
                  [
                    TopBarThemeExtensionDefault.from(Theme.of(context))
                        .titleStyle,
                    context.themeExtension<TopBarThemeExtension>()?.titleStyle,
                    title?.style,
                  ],
                  combine: (value, element) => value?.merge(element),
                ),
                gradientColors: const MultiColor([]),
              ),
              leading: leading,
              actions: actions,
              centerTitle: centerTitle ?? false,
            ),
            if (expandedWidget != null)
              ...ListTile.divideTiles(
                color: ThemeHelper.getElement<Color, Color>(
                  [
                    context
                        .themeExtension<TopBarThemeExtension>()
                        ?.dividerColor,
                    TopBarThemeExtensionDefault.from(Theme.of(context))
                        .dividerColor,
                  ],
                ),
                context: context,
                tiles: expandedWidget!,
              ),
          ],
        ),
      );
}
