// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/wyatt_ui_kit.dart';

class FileSelectionButtonThemeResolver extends ThemeResolver<
    FileSelectionButtonStyle, FileSelectionButtonThemeExtension, ButtonState> {
  const FileSelectionButtonThemeResolver({
    required this.customStyleFn,
  });

  @override
  FileSelectionButtonStyle? computeExtensionValueFn(
    BuildContext context,
    FileSelectionButtonThemeExtension? themeExtension, {
    ButtonState? extra,
  }) {
    FileSelectionButtonStyle? style;
    switch (extra?.state) {
      case ControlState.disabled:
        style = themeExtension?.disabledStyle;
        break;
      case ControlState.focused:
        style = themeExtension?.focusedStyle;
        break;
      case ControlState.hovered:
        style = themeExtension?.hoveredStyle;
        break;
      case ControlState.tapped:
        style = themeExtension?.tappedStyle;
        break;
      case null:
      case ControlState.normal:
        style = themeExtension?.normalStyle;
        break;
    }

    if (extra?.isInvalid ?? false) {
      style = themeExtension?.invalidStyle;
    }
    if (extra?.isSelected ?? false) {
      style = themeExtension?.selectedStyle;
    }

    return style;
  }

  @override
  FileSelectionButtonThemeExtension? getDefaultExtension(
    BuildContext context,
  ) =>
      FileSelectionButtonThemeExtensionDefault.from(Theme.of(context));

  @override
  final FileSelectionButtonStyle? Function(
    BuildContext context, {
    ButtonState? extra,
  }) customStyleFn;
}
